open Array
open Util
open Data

let sign x = if x < 0. then -1. else 1.

let dotExample v x n =
  let _P = dat_P x n 0 in
  let rec dot p acc =
    if p >= _P then acc 
    else if dat_i x n 0 p >= A1.dim v then dot (p+1) acc
    else dot (p+1) (acc +. dat_v x n 0 p *. v.{dat_i x n 0 p}) in
    dot 0 v.{0}

let exampleNorm x n =
  let _P = dat_P x n 0 in
  let rec dot p acc =
    if p >= _P then acc 
    else dot (p+1) (acc +. dat_v x n 0 p *. dat_v x n 0 p) in
    dot 0 0.

let compute_norms x = init_ba (dat_N x) (exampleNorm x)

let optimize_alma quiet alpha x xN y k w wNorm =
  let _N = dat_N x in
  let _B = 1. /. alpha in
  let _C = sqrt 2. in
    for n = 0 to _N - 1 do
      let gamma_k = _B /. sqrt (float_of_int !k) in
      let wtx     = dotExample w x n /. xN.{n} in
      let yn      = if y.(n) <= 0.5 then -1. else 1. in
        (*if not useY then Printf.fprintf stderr "%g %g %g\n" wtx yn ((1.-.alpha)*.gamma_k); *)
        if yn *. wtx <= (1. -. alpha) *. gamma_k then
          let eta_k = _C /. sqrt (float_of_int !k) in
            for f = 0 to dat_P x n 0 - 1 do
              let old_w = w.{dat_i x n 0 f} in
              let new_w = old_w +. yn *. eta_k *. dat_v x n 0 f /. xN.{n} in
                w.{dat_i x n 0 f} <- new_w;
                wNorm := !wNorm -. old_w *. old_w +. new_w *. new_w;
            done;
          let old_bias = w.{0} in
          let new_bias = old_bias +. yn *. eta_k /. xN.{n} in
            w.{0} <- new_bias;
            wNorm := !wNorm -. old_bias *. old_bias +. new_bias *. new_bias;
            if !wNorm > 1. then (
              for f = 0 to A1.dim w - 1 do
                w.{f} <- w.{f} /. !wNorm;
              done;
              wNorm := 1.;
            );
            incr k;
    done;
    ()
  

let my_print_info quiet err (tM : bigarr) tw ty (dM : bigarr) dw dy (sM : bigarr) sw sy =
  let de = err dM dw dy in
    if not quiet then (
      Printf.fprintf stderr " er %5.5f" (err tM tw ty);
      if length dy > 0 then Printf.fprintf stderr " der %5.5f" de;
      if length sy > 0 then Printf.fprintf stderr " ter %5.5f" (err sM sw sy);
    );
    de

let optimize quiet maxi alpha x y xDe yDe xTe yTe init_model ssData =
  let _N = dat_N x in
  let _F = dat_F x in

  let semi = match ssData with None -> None | Some (xSe,ySe) -> Some (xSe,ySe,compute_norms xSe) in

  let xN = compute_norms x in

  let w  = make_ba _F 0. in let wNorm = ref 0. in
  let or_empty x n = if n >= length x then [||] else x.(n) in
  let k = ref 1 in

  let compute_err mass xx yy =
    let total_mass = ref 0. in
    let total_score = ref 0. in
      for n = 0 to dat_N xx - 1 do
        let wtx     = dotExample w xx n in
        let yn      = if yy.(n) <= 0.5 then -1. else 1. in
          total_mass := !total_mass +. (if A1.dim mass >= n then mass.{n} else 1.);
          if yn *. wtx > 0. then
            total_score := !total_score +. (if A1.dim mass >= n then mass.{n} else 1.);
      done;
      (!total_mass -. !total_score) /. !total_mass in


    (match init_model with None -> () | Some wi -> 
       for f = 0 to min (x.datav_numfeat-1) (length wi-1) do
         w.{f} <- wi.(f);
         wNorm := !wNorm +. w.{f} *. w.{f};
       done);

    for iter=1 to maxi do
      if not quiet then printIter iter maxi;
      optimize_alma quiet alpha x xN y k w wNorm;
      my_print_info quiet compute_err x.datav_weights x y xDe.datav_weights xDe yDe xTe.datav_weights xTe yTe;
      if not quiet then Printf.fprintf stderr "\n";

      if iter > 15 then 
      (match semi with None -> () | Some (xSe, ySe, xSeN) ->
         if not quiet then printIter iter maxi;
         let yGuess = init (dat_N xSe) (fun n -> sign (dotExample w xSe n)) in
           optimize_alma quiet alpha xSe xSeN yGuess k w wNorm;
           my_print_info quiet compute_err x.datav_weights x y xDe.datav_weights xDe yDe xTe.datav_weights xTe yTe;
           if not quiet then Printf.fprintf stderr "\n");
         
    done;
    w

